#include <avr/io.h>
#include <util/delay.h>
#include "glcd.h"
//#include "grafik.h"

#define F_CPU 16000000UL  // 4 MHz
#define SPIPORT PORTB
#define SPIDDR DDRB
#define CS 5
#define CLK 1
#define SDA 2
#define RESET 6
#define cbi(reg, bit) (reg&=~(1<<bit))
#define sbi(reg, bit) (reg|= (1<<bit))
#define CS0 cbi(SPIPORT,CS);
#define CS1 sbi(SPIPORT,CS);
#define CLK0 cbi(SPIPORT,CLK);
#define CLK1 sbi(SPIPORT,CLK);
#define SDA0 cbi(SPIPORT,SDA);
#define SDA1 sbi(SPIPORT,SDA);
#define RESET0 cbi(SPIPORT,RESET);
#define RESET1 sbi(SPIPORT,RESET);
#define byte unsigned char
byte n=0;
byte s1,s2;
byte r,g,b;


//Funktionen
void lcdinit(void);
void sendCMD(byte cmd);
void sendData(byte cmd);
void shiftBits(byte b);
void LCD_put_pixel(unsigned char color, unsigned char x, unsigned char y);
void lcd_box(int lx1,int lx2,int ly1,int ly2,unsigned char color);
void clsdsp(unsigned char color);
void Lcd_window(int lx1,int lx2,int ly1,int ly2);
int pixel_lcd(int lx1,int lx2,int ly1,int ly2);
void grafik_lcd(int lx1,int lx2,int ly1,int ly2,unsigned char bild[]);


//DISPLAY TIMING
void waitms(int ms) {
  int i;
  for (i=0;i<ms;i++) _delay_ms(1);
}



//INIT DISPLAY
void lcdinit (void) 
{

  SPIDDR=(1<<SDA)|(1<<CLK)|(1<<CS)|(1<<RESET); //Port-Direction Setup


  
  int i;

  CS0
  SDA0
  CLK1

  RESET1
  RESET0
  waitms(10);
  RESET1

  CLK1
  SDA1
  CLK1

  sendCMD(0x01);
  waitms(10);
  sendCMD(0x11);
  waitms(40);
  sendCMD(0x29);
  waitms(40);
  sendCMD(0x3a);
  sendData(0x02);
  sendCMD(0x20);
  sendCMD(0x36);
  sendData(0x00);
  sendCMD(0x2d);

//FARBTABELLE 254
unsigned char lcdinitd[]={0x00 , 0x02 , 0x04 , 0x06 , 0x09 , 0x0B , 0x0D , 0x0F , 0x00 , 0x02 , 0x04 , 0x06 , 0x09 , 0x0B , 0x0D , 0x0F , 0x00 , 0x04 , 0x0B , 0x0F};

for (i=0;i<22;i++)
{
sendData(lcdinitd[i]);

}
//DISPLAY LSCHEN
 clsdsp(weiss);  

}


//Display Clear komplett
void clsdsp(unsigned char color)
{  
lcd_box(0,131,0,131,color);
}


void grafik_lcd(int lx1,int lx2,int ly1,int ly2,unsigned char bild[])
{
int i;
int pixel;
Lcd_window(lx1,lx2,ly1,ly2);
pixel=pixel_lcd(lx1,lx2,ly1,ly2);
for(i=0;i<pixel;i++)
{
sendData(bild);
}



}






//Display Fenster definieren
void lcd_box(int lx1,int lx2,int ly1,int ly2,unsigned char color)
{
Lcd_window(lx1,lx2,ly1,ly2);
int px;
int py;
int pixel;
int i;
   px = lx2 - lx1;
   px++; 
   py = ly2 - ly1;
   py++;
   pixel = px * py;
   for (i= 0;i<=pixel;i++)
   {
   sendData(color);

   }



}


//Fenster AdressBereich setzen
void Lcd_window(int lx1,int lx2,int ly1,int ly2)
{  sendCMD(0x2A);
   sendData(lx1);
   sendData(lx2);
   sendCMD(0x2B);
   sendData(ly1);
   sendData(ly2);
   sendCMD(0x2C);
  
}

void shiftBits(byte b) {

  CLK0
  if ((b&128)!=0) SDA1 else SDA0
  CLK1

  CLK0
  if ((b&64)!=0) SDA1 else SDA0
  CLK1

  CLK0
  if ((b&32)!=0) SDA1 else SDA0
  CLK1

  CLK0
  if ((b&16)!=0) SDA1 else SDA0
  CLK1

  CLK0
  if ((b&8)!=0) SDA1 else SDA0
  CLK1

  CLK0
  if ((b&4)!=0) SDA1 else SDA0
  CLK1

  CLK0
  if ((b&2)!=0) SDA1 else SDA0
  CLK1

  CLK0
  if ((b&1)!=0) SDA1 else SDA0
  CLK1

}


//send data
void sendData(byte data) {

  CLK0
  SDA1                                                 //1 for param
  CLK1

  shiftBits(data);
}

//send cmd
void sendCMD(byte data) {

  CLK0
  SDA0                                                 //1 for cmd
  CLK1

  shiftBits(data);
}

int pixel_lcd(int lx1,int lx2,int ly1,int ly2)
{
int px;
int py;
px = lx2 - lx1;
   px++; 
   py = ly2 - ly1;
   py++;
   return( px * py);


}
